pub struct AckNoop;Expand description
Acknowledgment handler that performs no operation.
This can be used in scenarios where an acknowledgment callback is required by the interface but no actual acknowledgement should be sent to the message broker.