AckNoop

Struct AckNoop 

pub struct AckNoop;
Acknowledgment handler that performs no operation.

This can be used in scenarios where an acknowledgment callback is required by the interface but no actual acknowledgement should be sent to the message broker.